@@ -11,6 +11,7 @@ setAs("ddenseMatrix", "dgeMatrix",
 	  else from
       })
 
+
 ## d(ouble) to l(ogical):
 setAs("dgeMatrix", "lgeMatrix", function(from) d2l_Matrix(from, "dgeMatrix"))
 setAs("dsyMatrix", "lsyMatrix", function(from) d2l_Matrix(from, "dsyMatrix"))
@@ -30,13 +31,14 @@ setAs("dgeMatrix", "dgCMatrix",
       function(from) .Call(dense_to_Csparse, from))
 
 setAs("matrix", "CsparseMatrix",
-      function(from) {
-	    if(is.numeric(from))
-		.Call(dense_to_Csparse, .Call(dup_mMatrix_as_dgeMatrix, from))
-	    else if(is.logical(from)) ## FIXME: this works, but maybe wastefully
-                as(Matrix(from, sparse=TRUE), "CsparseMatrix")
-	    else stop('not-yet-implemented coercion to "CsparseMatrix"')
-      })
+      function(from) .Call(dense_to_Csparse, from))
+##       function(from) {
+## 	    if(is.numeric(from))
+## 		.Call(dense_to_Csparse, .Call(dup_mMatrix_as_dgeMatrix, from))
+## 	    else if(is.logical(from)) ## FIXME: this works, but maybe wastefully
+##                 as(Matrix(from, sparse=TRUE), "CsparseMatrix")
+## 	    else stop('not-yet-implemented coercion to "CsparseMatrix"')
+##       })
 
 
 ## special case needed in the Matrix function
@@ -60,35 +62,35 @@ setMethod("as.numeric", signature(x = "ddenseMatrix"),
 ## dgeMatrix. Methods for special forms override these.
 
 setMethod("norm", signature(x = "ddenseMatrix", type = "missing"),
-          function(x, type, ...) callGeneric(as(x, "dgeMatrix")))
+	  function(x, type, ...) norm(as(x, "dgeMatrix")))
 
 setMethod("norm", signature(x = "ddenseMatrix", type = "character"),
-          function(x, type, ...) callGeneric(as(x, "dgeMatrix"), type))
+	  function(x, type, ...) norm(as(x, "dgeMatrix"), type))
 
 setMethod("rcond", signature(x = "ddenseMatrix", type = "missing"),
-          function(x, type, ...) callGeneric(as(x, "dgeMatrix")))
+	  function(x, type, ...) rcond(as(x, "dgeMatrix")))
 
 setMethod("rcond", signature(x = "ddenseMatrix", type = "character"),
-          function(x, type, ...) callGeneric(as(x, "dgeMatrix"), type))
+	  function(x, type, ...) rcond(as(x, "dgeMatrix"), type))
 
 ## Not really useful; now require *identical* class for result:
 ## setMethod("t", signature(x = "ddenseMatrix"),
 ## 	  function(x) callGeneric(as(x, "dgeMatrix")))
 
 setMethod("tcrossprod", signature(x = "ddenseMatrix", y = "missing"),
-	  function(x, y = NULL) callGeneric(as(x, "dgeMatrix")))
+	  function(x, y = NULL) tcrossprod(as(x, "dgeMatrix")))
 
 setMethod("crossprod", signature(x = "ddenseMatrix", y = "missing"),
-          function(x, y = NULL) callGeneric(as(x, "dgeMatrix")))
+          function(x, y = NULL) crossprod(as(x, "dgeMatrix")))
 
 setMethod("diag", signature(x = "ddenseMatrix"),
-          function(x, nrow, ncol) callGeneric(as(x, "dgeMatrix")))
+          function(x, nrow, ncol) diag(as(x, "dgeMatrix")))
 
 setMethod("solve", signature(a = "ddenseMatrix", b = "missing"),
-          function(a, b, ...) callGeneric(as(a, "dgeMatrix")))
+          function(a, b, ...) solve(as(a, "dgeMatrix")))
 
 setMethod("solve", signature(a = "ddenseMatrix", b = "ANY"),
-          function(a, b, ...) callGeneric(as(a, "dgeMatrix"), b))
+          function(a, b, ...) solve(as(a, "dgeMatrix"), b))
 
 ## General method for dense matrix multiplication in case specific methods
 ## have not been defined.
@@ -98,65 +100,66 @@ setMethod("%*%", signature(x = "ddenseMatrix", y = "ddenseMatrix"),
           valueClass = "dgeMatrix")
 
 setMethod("lu", signature(x = "ddenseMatrix"),
-          function(x, ...) callGeneric(as(x, "dgeMatrix")))
+          function(x, ...) lu(as(x, "dgeMatrix")))
 
 setMethod("chol", signature(x = "ddenseMatrix", pivot = "ANY"), cholMat)
 
 setMethod("determinant", signature(x = "ddenseMatrix", logarithm = "missing"),
-          function(x, logarithm, ...) callGeneric(as(x, "dgeMatrix")))
+	  function(x, logarithm, ...) determinant(as(x, "dgeMatrix")))
 
 setMethod("determinant", signature(x = "ddenseMatrix", logarithm = "logical"),
-          function(x, logarithm, ...)
-          callGeneric(as(x, "dgeMatrix"), logarithm))
+	  function(x, logarithm, ...)
+	  determinant(as(x, "dgeMatrix"), logarithm))
 
 ## now done for "dMatrix":
 ## setMethod("expm", signature(x = "ddenseMatrix"),
 ##           function(x) callGeneric(as(x, "dgeMatrix")))
 
-setMethod("Schur", signature(x = "ddenseMatrix", vectors = "missing"),
-          function(x, vectors, ...) callGeneric(as(x, "dgeMatrix")))
-
-setMethod("Schur", signature(x = "ddenseMatrix", vectors = "logical"),
-          function(x, vectors, ...) callGeneric(as(x, "dgeMatrix"), vectors))
-
-
 setMethod("Math",
           signature(x = "ddenseMatrix"),
           function(x) callGeneric(as(x, "dgeMatrix")))
 
 
-### FIXME: band() et al should be extended from "ddense" to "dense" !
-###        However, needs much work to generalize dup_mMatrix_as_dgeMatrix()
 
+.trilDense <- function(x, k = 0, ...) {
+    k <- as.integer(k[1])
+    d <- dim(x)
+    stopifnot(-d[1] <= k, k <= d[1]) # had k <= 0
+    ## returns "lower triangular" if k <= 0 && sqr
+    .Call(dense_band, x, -d[1], k)
+}
 ## NB: have extra tril(), triu() methods for symmetric ["dsy" and "dsp"] and
 ##     for triangular ["dtr" and "dtp"]
-setMethod("tril", "ddenseMatrix",
-	  function(x, k = 0, ...) {
-	      k <- as.integer(k[1])
-	      dd <- dim(x); sqr <- dd[1] == dd[2]
-	      stopifnot(-dd[1] <= k, k <= dd[1]) # had k <= 0
-	      ## returns "lower triangular" if k <= 0 && sqr
-	      .Call(ddense_band, x, -dd[1], k)
-	  })
+setMethod("tril", "denseMatrix", .trilDense)
+setMethod("tril",      "matrix", .trilDense)
 
-setMethod("triu", "ddenseMatrix",
-	  function(x, k = 0, ...) {
-	      k <- as.integer(k[1])
-	      dd <- dim(x); sqr <- dd[1] == dd[2]
-	      stopifnot(-dd[1] <= k, k <= dd[1]) # had k >= 0
-	      ## returns "upper triangular" if k >= 0
-	      .Call(ddense_band, x, k, dd[2])
-	  })
+.triuDense <- function(x, k = 0, ...) {
+    k <- as.integer(k[1])
+    d <- dim(x)
+    stopifnot(-d[1] <= k, k <= d[1]) # had k >= 0
+    ## returns "upper triangular" if k >= 0
+    .Call(dense_band, x, k, d[2])
+}
+setMethod("triu", "denseMatrix", .triuDense)
+setMethod("triu",      "matrix", .triuDense)
+
+.bandDense <- function(x, k1, k2, ...) {
+    k1 <- as.integer(k1[1])
+    k2 <- as.integer(k2[1])
+    dd <- dim(x); sqr <- dd[1] == dd[2]
+    stopifnot(-dd[1] <= k1, k1 <= k2, k2 <= dd[1])
+    r <- .Call(dense_band, x, k1, k2)
+    if (sqr &&  k1 < 0 &&  k1 == -k2  && isSymmetric(x)) ## symmetric
+	forceSymmetric(r)
+    else
+	r
+}
+setMethod("band", "denseMatrix", .bandDense)
+setMethod("band",      "matrix", .bandDense)
+
+
+setMethod("symmpart", signature(x = "ddenseMatrix"),
+	  function(x) .Call(ddense_symmpart, x))
+setMethod("skewpart", signature(x = "ddenseMatrix"),
+	  function(x) .Call(ddense_skewpart, x))
 
-setMethod("band", "ddenseMatrix",
-	  function(x, k1, k2, ...) {
-	      k1 <- as.integer(k1[1])
-	      k2 <- as.integer(k2[1])
-	      dd <- dim(x); sqr <- dd[1] == dd[2]
-	      stopifnot(-dd[1] <= k1, k1 <= k2, k2 <= dd[1])
-	      r <- .Call(ddense_band, x, k1, k2)
-	      if (k1 < 0  &&  k1 == -k2  && isSymmetric(x)) ## symmetric
-		  as(r, paste(.M.kind(x), "syMatrix", sep=''))
-	      else
-		  r
-	  })
